# Created by: Martin Flanagan (FLNMAR011)
# Project: PyPiGame
# Desktop Application

import pygame
import time

from pygame.locals import *
from character import Character
from maze import Maze

class GameApp:

    window_width = 400
    window_length = 400
    size = (window_width, window_length)

    player_one = None
    #When the object is created this will run
    def __init__(self):
        self.is_running = True
        self.display = None
        self.player_img = None
        self.walls_img = None
        self.goal_img = None
        self.win_img = None
        self.player_one = Character()
        self.maze = Maze()

    #initialize the display properly and load all images needed
    def on_init(self):
        #Dealwith the display and the pygame framework
        pygame.init()
        self.display = pygame.display.set_mode(self.size)
        pygame.display.set_caption('PyPiGame')
        self.is_running = True
        self.clock = pygame.time.Clock()
        # Load images
        self.player_img = pygame.image.load("Player.png").convert()
        self.walls_img = pygame.image.load("Wall.png").convert()
        self.goal_img = pygame.image.load("Goal.png").convert()
        self.win_img = pygame.image.load("win.png").convert()

    def on_event(self, event):
        #Stop running the game loop if you decide to quit
        if event.type == QUIT:
            self.is_running = False

    def on_game_logic(self):
        if self.maze.check_collisions(self.player_one):
            self.player_one.moveBack()

    def on_loop_again(self):
        #Delaythe next iteration of the game so that we don't have infinite iterations of the game each second
        #Limit the number of frames per second to 60
        self.clock.tick(60)

    def on_render(self):
        #Clear screen first before adding drawing commands
        self.display.fill((0,0,0))
        #Draw the objects on the screen
        self.display.blit(self.player_img, (self.player_one.x, self.player_one.y))  #Player
        self.maze.draw_maze(self.display, self.walls_img, self.goal_img)            #Maze
        #Update the screen with the latest information about the game
        pygame.display.flip()

    def on_render_win(self):
        #Clear screen first before adding drawing commands
        self.display.fill((0,0,0))
        #Draw the objects on the screen
        self.maze.draw_win(self.display, self.win_img)            #Maze
        #Update the screen with the latest information about the game
        pygame.display.flip()

    def on_cleanup(self):
        #Make sure that the program closes properly after the game is closed
        pygame.quit()

    def on_execute(self):
        #Check if we can properly initiate the player, maze and the display
        if self.on_init() == False:
            self.is_running = False

        if self.is_running:
            self.maze.count_walls_for()
            self.maze.count_walls_while()
            self.maze.find_goal()

        #This is essentially the game Loop
        while (self.is_running):
            #1 - Get any event that has happened
            for event in pygame.event.get():
                if event.type == pygame.QUIT: #Quit button pressed
                    self.is_running = False
                elif event.type == pygame.KEYUP:
                    arrow_key = event.key
                    if (arrow_key == pygame.K_RIGHT):
                        self.player_one.moveRight()
                    if (arrow_key == pygame.K_LEFT):
                        self.player_one.moveLeft()
                    if (arrow_key == pygame.K_UP):
                        self.player_one.moveUp()
                    if (arrow_key == pygame.K_DOWN):
                        self.player_one.moveDown()
                    if (arrow_key == pygame.K_ESCAPE):
                        self.is_running = False

            #2 - Game logic (Check for any collisions)
            self.on_game_logic()
            #3 - Render anything that needs to be rendered
            if self.maze.win:
                self.on_render_win()
            else:
                self.on_render()
            #4 - delay the next iteration of the loop
            self.on_loop_again()
        #Any cleanup needed, this will take care of it
        self.on_cleanup()

if __name__ == '__main__':
    game = GameApp()
    game.on_execute()
